(ns com.aurellem.run.ram-display
  (:use (com.aurellem.gb saves gb-driver util constants
                         items vbm characters money
                         rlm-assembly))
  (:use (com.aurellem.run image util music title save-corruption
                          bootstrap-0 bootstrap-1))
  (:require clojure.string)
  (:import [com.aurellem.gb.gb_driver SaveState])

  (:import java.awt.image.BufferedImage)
  (:import java.io.File))

(defn accurate-memory-range
  ([^SaveState state start end]
     (map accurate-memory (range start end)))
  ([start end]
     (accurate-memory-range @current-state start end)))

(def video-ram-start 0x8000)

(def GB-memory-end 0x10000)

(defn ram-image
  ([^SaveState state]
     (let [width 128
           height 256
           image (BufferedImage. width height
                                 BufferedImage/TYPE_INT_RGB)
           data (vec
                 (accurate-memory-range
                  state video-ram-start GB-memory-end))]
       (dorun
        (map
         (fn [index]
           (let [gray (data index)]
             (.setRGB image (rem index width)
                      (int (/ index width))
                      (+
                       (bit-shift-left gray 16)
                       (bit-shift-left gray 8)
                       gray))))
         (range (count data))))
       image))
  ([] (ram-image @current-state)))

(defn write-ram-image!
  ([^SaveState state ^File target]
     (write-image! (ram-image state) target))
  ([^File target]
     (write-ram-image! @current-state target)))

(def test-ram-image
  (File. "/home/r/proj/vba-clojure/images/RAM-test.png"))
